@@ -5,6 +5,10 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
   have.bounds<-ctrl$have.bounds
   ctrl$have.bounds<-NULL ## or we get errors in optim()
   npar<-length(par)
+## 131027
+  if (length(lower) == 1) lower<-rep(lower,npar)
+  if (length(upper) == 1) upper<-rep(upper,npar)
+## end 131027
   nmeth<-length(method)
   pstring<-names(par)
   if (is.null(pstring)) {
@@ -17,12 +21,12 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
   row.names(ans.ret)<-method
   ans.details <- list()
   ansout <- NULL # ensure NULL if we have no parameters or no successes
+# if (is.null(ugr)) stop("Cannot have null ugr() in optimx.run")
+  numgrad<-FALSE
+  # 130924 -- need to fix -- some methods can handle null gradient
   for (i in 1:nmeth) { # loop over the methods
       meth <- method[i] # extract the method name
       conv <- -1 # indicate that we have not yet converged
-#      cat("optimx.run with method ",meth," ctrl:")
-#      tmp <- readline("CONTINUE")
-#     print(ctrl)
       # 20100608 - take care of polyalgorithms
       if (! is.null(itnmax) ) {
 	if (length(itnmax) == 1) {ctrl$maxit <- itnmax} # Note we will execute this FIRST
@@ -56,13 +60,8 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
       if (meth=="Nelder-Mead" || meth == "BFGS" || meth == "L-BFGS-B" || meth == "CG" || meth == "SANN") {
 #       if (meth == "SANN") mcontrol$maxit<-10000 # !! arbitrary for now 
         # Take care of methods   from optim(): Nelder-Mead, BFGS, L-BFGS-B, CG
-
-#        cat("optim() par:")
-#        print(par)
-
-
         time <- system.time(ans <- try(optim(par=par, fn=ufn, gr=ugr, lower=lower, upper=upper, 
-                method=meth, control=mcontrol, ...)))[1]
+                method=meth, control=mcontrol, ...), silent=TRUE))[1]
         # The time is the index=1 element of the system.time for the process, 
         # which is a 'try()' of the regular optim() function
         if (class(ans)[1] != "try-error") { 
@@ -114,6 +113,7 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
 		ans$value= ctrl$badval
 		ans$par<-rep(NA,npar)
         	ans$nitns<-NA # not used
+                ans$gevals<-NA ## ?? missing 130929
         }
         ans$convergence<-NULL
 ##	if (ctrl$maximize) {
@@ -123,6 +123,9 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
 ##		        print(ans)
 ##                }
 ##	}
+##          ans.ret[meth, ] <- c(ans$par, ans$value, ans$fevals, ans$gevals, ans$nitns,
+##                              ans$convcode, ans$kkt1, ans$kkt2, ans$xtimes)
+
       }  ## end if using nlminb
 ## --------------------------------------------
       else if (meth == "nlm") { # Use stats package nlm routine
@@ -249,13 +252,16 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
 ## --------------------------------------------
 ###### progress point #########
       else if (meth == "Rcgmin") { # Use Rcgmin routine (ignoring masks)
-	bdmsk<-rep(1,npar)
+##	bdmsk<-rep(1,npar)
+	bdmsk<-bmchk(par, lower=lower, upper=upper)$bdmsk ## 131027 removed
 	mcontrol$trace<-NULL
 	if (ctrl$trace>0) mcontrol$trace<-1
 	if (have.bounds) {
-   	   time <- system.time(ans <- try(Rcgminb(par=par, fn=ufn, gr=ugr, lower=lower, upper=upper, 
-		bdmsk=bdmsk, control=mcontrol, ...), silent=TRUE))[1]
+           if (is.null(ugr)) ugr<-"grfwd" ##JN
+   	   time <- system.time(ans <- try(Rcgminb(par=par, fn=ufn, gr=ugr, lower=lower,
+                upper=upper, bdmsk=bdmsk, control=mcontrol, ...), silent=TRUE))[1]
 	} else {
+           if (is.null(ugr)) ugr<-"grfwd" ##JN
    	   time <- system.time(ans <- try(Rcgminu(par=par, fn=ufn, gr=ugr, 
 		control=mcontrol, ...), silent=TRUE))[1]
 	}
@@ -279,18 +285,22 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
       }  ## end if using Rcgmin
 ## --------------------------------------------
 ###### progress point #########
-      else if (meth == "Rvmmin") { # Use Rvmmin routine (ignoring masks)
-	bdmsk<-rep(1,npar)
+      else if (meth == "Rvmmin") { # Use Rvmmin routine (ignoring masks??)
+##	bdmsk<-rep(1,npar) ## 131027
+	bdmsk<-bmchk(par, lower=lower, upper=upper)$bdmsk ## 131027 removed
 	mcontrol$trace<-NULL
 	if (ctrl$trace>0) mcontrol$trace<-1
 	if (have.bounds) {
-   	   time <- system.time(ans <- try(Rvmminb(par=par, fn=ufn, gr=ugr, lower=lower, upper=upper, 
-		bdmsk=bdmsk, control=mcontrol, ...), silent=TRUE))[1]
+           if (is.null(ugr)) ugr<-"grfwd" ##JN
+   	   time <- system.time(ans <- try(Rvmminb(par=par, fn=ufn, gr=ugr, lower=lower,
+                upper=upper, bdmsk=bdmsk, control=mcontrol, ...), silent=TRUE))[1]
 	} else {
+           if (is.null(ugr)) ugr<-"grfwd" ##JN
    	   time <- system.time(ans <- try(Rvmminu(par=par, fn=ufn, gr=ugr, 
 		control=mcontrol, ...), silent=TRUE))[1]
 	}
-        if ((class(ans)[1] != "try-error") && (ans$convergence==0)) {
+##        if ((class(ans)[1] != "try-error") && (ans$convergence==0)) {
+        if (class(ans)[1] != "try-error") {
 		ans$convcode <- ans$convergence
 	        ans$fevals<-ans$counts[1]
 	        ans$gevals<-ans$counts[2]
@@ -435,29 +445,12 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
          mcontrol$maximize<-NULL
          mcontrol$parscale<-NULL
          mcontrol$fnscale<-NULL
-#         cat("nmkb mcontrol\n")
-#        print(mcontrol)
-#        cat("nmkb par:")
-#        print(par)
-#         if (have.bounds) {
-#            time <- system.time(ans <- try(nmkb(par=par, fn=ufn, lower = lower, 
-#              upper = upper, control=mcontrol, ...), silent=TRUE))[1]
-#         } else {
-#            time <- system.time(ans <- try(nmk(par=par, fn=ufn, 
-#              control=mcontrol, ...), silent=TRUE))[1]
-#         }
          if (have.bounds) {
             time <- system.time(ans <- try(nmkb(par=par, fn=ufn, lower = lower, 
-              upper = upper, control=mcontrol, ...)))[1]
+              upper = upper, control=mcontrol, ...), silent=TRUE))[1]
          } else {
-## this worked when control=mcontrol did not
-##            time <- system.time(ans <- try(nmk(par=par, fn=ufn, 
-##              control=list(trace=TRUE), ...)))[1]
-## as.list did not work
-#           print(str(mcontrol))
-#           print(class(mcontrol))
             time <- system.time(ans <- try(nmk(par=par, fn=ufn, 
-              control=as.pairlist(mcontrol), ...)))[1]
+              control=mcontrol, ...), silent=TRUE))[1]
          }
          if (class(ans)[1] != "try-error") {
             ans$convcode <- ans$convergence
@@ -497,7 +490,8 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
          mcontrol$info<-FALSE # no trace printed
          if (mcontrol$trace > 0) {
             mcontrol$info<-TRUE # logical needed, not integer         
-         } else { mcontrol$trace<-FALSE }
+         } else { mcontrol$info<-FALSE }
+         mcontrol$trace=NULL
          mcontrol$usenumDeriv<-NULL
          mcontrol$maximize<-NULL
          mcontrol$parscale<-NULL
@@ -512,8 +506,13 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
             time <- system.time(ans <- try(hjkb(par=par, fn=ufn, lower = lower, 
                 upper = upper, control=mcontrol, ...), silent=TRUE))[1]
          } else {
+#            cat("call hjk with function ",deparse(substitute(ufn)),"\n")
+#            cat("F(par)=",ufn(par, ...), " for par =")
+#            print(par)
+#            print(mcontrol)
             time <- system.time(ans <- try(hjk(par=par, fn=ufn, 
                 control=mcontrol, ...), silent=TRUE))[1]
+#            print(ans)
          }
          if (class(ans)[1] != "try-error") {
             ans$convcode <- ans$convergence
@@ -562,15 +561,15 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
           if ((ctrl$kkt || hessian) && (ans$convcode != 9999)) {
               if (ctrl$trace>0) cat("Compute Hessian approximation at finish of ",method[i],"\n")
               if (!is.null(uhess)){ # check if we have analytic hessian 
-                 nhatend<-try(uhess(ans$par, ...))
+                 nhatend<-try(uhess(ans$par, ...), silent=TRUE)
                  if (class(nhatend) != "try-error") {
                     hessOK<-TRUE
                  }
               } else {
                  if (is.null(ugr)) {
-                     nhatend<-try(hessian(ufn, ans$par, ...)) # change 20100711
+                     nhatend<-try(hessian(ufn, ans$par, ...), silent=TRUE) # change 20100711
                  } else {
-                     nhatend<-try(jacobian(ugr,ans$par, ...)) # change 20100711
+                     nhatend<-try(jacobian(ugr,ans$par, ...), silent=TRUE) # change 20100711
                  } # numerical hessian at "solution"
                  if (class(nhatend) != "try-error") {
                     hessOK<-TRUE
@@ -582,9 +581,9 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
           if ((hessian || ctrl$kkt) && (ans$convcode != 9999)) {# avoid test when method failed
              if (ctrl$trace>0) cat("Compute gradient approximation at finish of ",method[i],"\n")
              if (is.null(ugr)) {
-                 ngatend<-try(grad(ufn, ans$par, ...)) # change 20100711
+                 ngatend<-try(grad(ufn, ans$par, ...), silent=TRUE) # change 20100711
              } else {
-                 ngatend<-try(ugr(ans$par, ...)) # Gradient at solution # change 20100711
+                 ngatend<-try(ugr(ans$par, ...), silent=TRUE) # Gradient at solution # change 20100711
              }
              if (class(ngatend) != "try-error") gradOK<-TRUE # 100215 had == rather than != here
              if ( (! gradOK) && (ctrl$trace>0)) cat("Gradient computation failure!\n") 
@@ -601,7 +600,7 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
                    nhatend[bset,] <-0
                    nhatend[, bset] <-0 # and the Hessian
                    if (!isSymmetric(nhatend, tol=sqrt(.Machine$double.eps))) {
-                      hessOK<-FALSE
+                      # hessOK<-FALSE ## Assume we will keep hessian after symmetrizing
                       asym <- sum(abs(t(nhatend) - nhatend))/sum(abs(nhatend))
                       asw <- paste("Hessian is reported non-symmetric with asymmetry ratio ", 
                       asym, sep = "")
@@ -612,7 +611,7 @@ optimx.run <- function(par, ufn, ugr=NULL, uhess=NULL, lower=-Inf, upper=Inf,
                       if (ctrl$dowarn) warning("Hessian forced symmetric", call. = FALSE)
                       nhatend <- 0.5 * (t(nhatend) + nhatend)
                    }  # end symmetry test
-                   hev<- try(eigen(nhatend)$values) # 091215 use try in case of trouble
+                   hev<- try(eigen(nhatend)$values, silent=TRUE) # 091215 use try in case of trouble
                    if (ctrl$kkt){
    	              if (class(hev) != "try-error") {# answers are OK, check Hessian properties
                          if (any(is.complex(hev))){
